#!usr/bin/python
# Draft for ToxAnalyzer in Python - Heatmaps
# (c) 2019 - Daniel R. Rodrigues

import urllib.request as ur # library for getting data from the web
import sys # to raise a message to the user and then break the program
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sb

compound = input("Type the name of a chemical compound to get its interactions: ")

url = "http://ctdbase.org/tools/batchQuery.go?inputType=chem&inputTerms="+compound+"&report=cgixns&format=tsv"

ctd_table = ur.urlopen(url)

ctd_table_opened = ctd_table.read().decode("utf-8")
'''
using decode() here because urls come in as bytes and I need to work
with strings, so a conversion is necessary...
https://stackoverflow.com/q/49731550
https://www.youtube.com/watch?v=5GzVNi0oTxQ
'''


if "Object not found" in ctd_table_opened:
    sys.exit("The database could not find the chemical you were looking for."
    "\nEither enter a valid chemical name or try again!")

# getting the table from CTD
pandas_table = pd.read_csv(url, sep="\t")

# removing the cluttering
table = pandas_table.drop(columns=["# Input", "ChemicalID", "CasRN", "Interaction"])

# dropping missing lines
table = table.dropna()

# ordering about interactions
interactions_table = table.groupby("InteractionActions")["GeneSymbol"].count()
interactions_table = interactions_table.reset_index(name='InteractionCount') # not letting white spaces in the columns


# splitting the data into smaller sets
affect_data = interactions_table[interactions_table['InteractionActions'].str.startswith("affects")]
decrease_data = interactions_table[interactions_table['InteractionActions'].str.startswith("decreases")]
increase_data = interactions_table[interactions_table['InteractionActions'].str.startswith("increases")]

# removing data that is lower than 3.5% of its data
# 3.5% is an arbitrary number
sum_affected = int((affect_data['InteractionCount'].sum()) * 0.035)
sum_decreased = int((increase_data['InteractionCount'].sum()) * 0.035)
sum_increased = int((decrease_data['InteractionCount'].sum()) * 0.035)

# generating the smaller datasets
affect_data = affect_data[affect_data['InteractionCount'] > sum_affected]
decrease_data = decrease_data[decrease_data['InteractionCount'] > sum_decreased]
increase_data = increase_data[increase_data['InteractionCount'] > sum_increased]


# generating the subplots
plt.subplot(3, 1, 1)
a = sb.barplot(x="InteractionCount", y="InteractionActions", data=affect_data)
#a.set_xticklabels(a.get_xticklabels(), rotation=90)
plt.xlabel('')
plt.ylabel(' ')
plt.title("Affecting Interactions of %s" %compound)
#
#
plt.subplot(3, 1, 2)
b = sb.barplot(x="InteractionCount", y="InteractionActions", data=decrease_data)
#b.set_xticklabels(b.get_xticklabels(), rotation=90)
plt.xlabel('')
plt.ylabel('')
plt.title("Decreasing Interactions of %s" %compound)
#
#
plt.subplot(3, 1, 3)
c = sb.barplot(x="InteractionCount", y="InteractionActions", data=increase_data)
#c.set_xticklabels(c.get_xticklabels(), rotation=90)
plt.xlabel('')
plt.ylabel('')
plt.title("Increasing Interactions of %s" %compound)
plt.tight_layout()
plt.show()

# interactions by organism (NO CUTOFFS HERE!!!)
organism_table = table.groupby("Organism")["GeneSymbol"].count()
organism_table = organism_table.reset_index(name='InteractionCount') # not letting white spaces in the columns

#plotting those interactions
d = sb.barplot(x="InteractionCount", y="Organism", data=organism_table)
#a.set_xticklabels(a.get_xticklabels(), rotation=90)
plt.xlabel('')
plt.ylabel(' ')
plt.title("Interactions of %s by organism" %compound)
plt.show()

# plt.figure()
# plt.subplot(3, 1, 1)
# plt.barplot("InteractionActions", "InteractionCount", data=affect_data)
# plt.title("Affecting Interactions")
#
# plt.subplot(3, 1, 2)
# plt.plot("InteractionActions", "InteractionCount", data=decrease_data)
# plt.title("Decreasing Interactions")
#
# plt.subplot(3, 1, 3)
# plt.plot("InteractionActions", "InteractionCount", data=increase_data)
# plt.title("Increasing Interactions")
# plt.show()
#
