import urllib.request as ur # library for getting data from the web
import sys # to raise a message to the user and then break the program
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sb
from io import StringIO


compound = input("Type the name of a chemical compound to get its interactions: ")


url = "http://ctdbase.org/tools/batchQuery.go?inputType=chem&inputTerms="+compound+"&report=cgixns&format=tsv"


request = ur.Request(url, headers={'User-Agent': 'Mozilla/5.0'})


ctd_table = ur.urlopen(request)



ctd_table_opened = ctd_table.read().decode("utf-8")

ctd_table_opened = StringIO(ctd_table_opened)

'''
using decode() here because urls come in as bytes and I need to work
with strings, so a conversion is necessary...
https://stackoverflow.com/q/49731550
https://www.youtube.com/watch?v=5GzVNi0oTxQ
'''


if "Object not found" in ctd_table_opened:
    sys.exit("The database could not find the chemical you were looking for."
    "\nEither enter a valid chemical name or try again!")

# getting the table from CTD
pandas_table = pd.read_csv(ctd_table_opened, delim_whitespace=True)

# removing the cluttering
table = pandas_table.drop(columns=["# Input", "ChemicalID", "CasRN", "Interaction"])

# dropping missing lines
table = table.dropna()

# ordering about interactions

interactions_table = table.groupby(["Organism", "InteractionActions"]).size()
interactions_table = interactions_table.reset_index(name='InteractionCount') # not letting white spaces in the columns

# renaming rows
# # from https://stackoverflow.com/q/53499792
d = {"affects": "Affecting", "decreases": "Decreasing", "increases": "Increasing"}

for k, v in d.items():
    interactions_table.loc[interactions_table['InteractionActions'].str.startswith(k, na=False), 'InteractionActions'] = v


interactions_table = interactions_table.groupby(["Organism", "InteractionActions"]).sum(axis=0)

# from https://pstblog.com/2016/10/04/stacked-charts
colors = ["#FF9F1C", "#E71D36","#2EC4B6"]

pivot_df = interactions_table.pivot_table(index='Organism', columns='InteractionActions', values='InteractionCount')
pivot_df.loc[:,['Affecting','Decreasing', 'Increasing']].plot.bar(stacked=True, color=colors, figsize=(10,7)).legend()

plt.show()
